<?php

namespace App\Modules\Admin\Services;

use App\Base\BaseService;
use App\Models\Goods\Goods;
use App\Models\Goods\GoodsCategoryMap;
use App\Models\Goods\GoodsSpecMap;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;

class GoodsService extends BaseService
{
    public function paginate($params)
    {
        $p = Goods::query()->when($params['status'], function (Builder $query) use ($params) {
            return $query->where("status", $params['status']);
        })->when($params['name'], function (Builder $query) use ($params) {
            return $query->where("name", "like", "%{$params['name']}%");
        })->orderByDesc("id")->paginate($params['page_size']);

        return [
            "total" => $p->total(),
            "page_total" => $p->lastPage(),
            "list" => array_map(function (Goods $model) {

                return [
                    "id" => $model->id,
                    "name" => $model->name,
                    "thumb" => Arr::first($model->image_list),
                    "weight" => $model->weight,
                    "status" => $model->status,
                    "created_at" => $model->created_at->format("Y-m-d H:i:s"),
                ];
            }, $p->items()),
        ];
    }

    public function store($params)
    {
        if ($id = Arr::get($params, "id", 0)) {
            $goods = Goods::findOrFail($id);
        } else {
            $goods = new Goods();
        }

        $specIdArr = array_values(Arr::sort(Arr::get($params, "spec", [])));

        DB::transaction(function () use ($params, $goods, $specIdArr) {
            $goods->image_list = Arr::get($params, "image_list", []);
            $goods->sn = $params['sn'] ?? "";
            $goods->url_3d = $params['url_3d'] ?? "";
            $goods->spec = $specIdArr;
            $goods->name = Arr::get($params, "name", []);
            $goods->weight = Arr::get($params, "weight", 0);
            $goods->desc_html = Arr::get($params, "desc_html", "");
            $goods->spec_attr_html = Arr::get($params, "spec_attr_html", "");
            $goods->faq_html = Arr::get($params, "faq_html", "");
            $goods->save();
            $categoryArr = Arr::get($params, "category", []);
            GoodsCategoryMap::where("goods_id", $goods->id)->whereNotIn("category_id", Arr::pluck($categoryArr, "id"))->delete();
            foreach ($categoryArr as $c) {
                /**
                 * [
                 * "id" => 1
                 * "is_public" => 1
                 * ]
                 */
                GoodsCategoryMap::updateOrCreate([
                    "goods_id" => $goods->id,
                    "category_id" => $c['id'],
                    'is_public' => $c['is_public'],
                ]);
            }
            $specMap = Arr::get($params, "spec_map", []);
            $doNotDelIdArr = [];
            foreach ($specMap as $c) {

                $model = GoodsSpecMap::where("goods_id", $goods->id)->where("spec_id", $c['spec_id'])->first();
                if (is_null($model)) {
                    $model = new GoodsSpecMap();
                    $model->goods_id = $goods->id;
                    $model->spec_id = $c['spec_id'];
                }

                $model->value = $c['value'];

                $model->save();
                $doNotDelIdArr[] = $model->id;
            }

            GoodsSpecMap::where("goods_id", $goods->id)->whereNotIn("id", $doNotDelIdArr)->delete();
        });

    }

    public function info($params)
    {
        $id = $params['id'];
        $model = Goods::findOrFail($id);

        $return = [
            "id" => $model->id,
            "name" => $model->name,
            "image_list" => $model->image_list,
            "category" => $model->map->map(function (GoodsCategoryMap $map) {
                return [
                    "id" => $map->category->id,
                    "is_public" => $map->is_public,
                    "name" => $map->category->name,
                ];
            }),
            "spec" => $model->spec,

            "weight" => $model->weight,
            "desc_html" => $model->desc_html,
            "spec_attr_html" => $model->spec_attr_html,
            "faq_html" => $model->faq_html,
            "sn" => $model->sn,
            "url_3d" => $model->url_3d,
            "spec_map" => $model->specMap->map(function (GoodsSpecMap $map) {
                return [
                    "id" => $map->id,
                    "spec_id" => $map->spec_id,
                    "value" => $map->value,
                    "name" => $map->spec->name,
                ];
            }),
        ];

        return $return;
    }
}
